/*
 * Copyright (C) 2015 Stratio (http://stratio.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.stratio.sparta.plugin.input.kafka

import java.io.{Serializable => JSerializable}

import com.stratio.sparta.sdk.ValidatingPropertyMap._

trait KafkaBase {

  def getZkConnectionConfs(
                            properties: Map[String, JSerializable],
                            key: String,
                            defaultHost: String,
                            defaultPort: String,
                            zookeeperPath: String):
  (String, String) = {
    val hostList = properties.getMapFromJsoneyString(key)
    val hosts = (for {
      hostConfig <- hostList
      host =  hostConfig.get("host").getOrElse(defaultHost)
      port =  hostConfig.get("port").getOrElse(defaultPort)
    } yield s"$host:$port").mkString(",")

    val fullConnectionPath = if (zookeeperPath.isEmpty) hosts else s"$hosts/$zookeeperPath"
    val validatedZookeeperPath = fullConnectionPath.replaceAll("//", "/")

    (key.toString, validatedZookeeperPath)
  }

}
